@@ -26,17 +26,19 @@
  * SUCH DAMAGE.
  */
 
+#include <sys/stat.h>
 #include <sys/statvfs.h>
 
 #include <ctype.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "common.h"
 #include "dhcpcd.h"
 #include "if-options.h"
-#include "logerr.h"
 
 const char *
 hwaddr_ntoa(const void *hwaddr, size_t hwlen, char *buf, size_t buflen)
@@ -44,7 +46,7 @@ hwaddr_ntoa(const void *hwaddr, size_t hwlen, char *buf, size_t buflen)
 	const unsigned char *hp, *ep;
 	char *p;
 
-	if (buf == NULL)
+	if (buf == NULL || hwlen == 0)
 		return NULL;
 
 	if (hwlen * 3 > buflen) {
@@ -103,38 +105,97 @@ hwaddr_aton(uint8_t *buffer, const char *addr)
 	return len;
 }
 
-size_t
-read_hwaddr_aton(uint8_t **data, const char *path)
+ssize_t
+readfile(const char *file, void *data, size_t len)
 {
-	FILE *fp;
-	char *buf;
-	size_t buf_len, len;
+	int fd;
+	ssize_t bytes;
 
-	if ((fp = fopen(path, "r")) == NULL)
-		return 0;
+	fd = open(file, O_RDONLY);
+	if (fd == -1)
+		return -1;
+	bytes = read(fd, data, len);
+	close(fd);
+	if ((size_t)bytes == len) {
+		errno = ENOBUFS;
+		return -1;
+	}
+	return bytes;
+}
 
-	buf = NULL;
-	buf_len = len = 0;
-	*data = NULL;
-	while (getline(&buf, &buf_len, fp) != -1) {
-		if ((len = hwaddr_aton(NULL, buf)) != 0) {
-			if (buf_len >= len)
-				*data = (uint8_t *)buf;
-			else {
-				if ((*data = malloc(len)) == NULL)
-					len = 0;
-			}
-			if (len != 0)
-				(void)hwaddr_aton(*data, buf);
-			if (buf_len < len)
-				free(buf);
+ssize_t
+writefile(const char *file, mode_t mode, const void *data, size_t len)
+{
+	int fd;
+	ssize_t bytes;
+
+	fd = open(file, O_WRONLY | O_CREAT | O_TRUNC, mode);
+	if (fd == -1)
+		return -1;
+	bytes = write(fd, data, len);
+	close(fd);
+	return bytes;
+}
+
+int
+filemtime(const char *file, time_t *time)
+{
+	struct stat st;
+
+	if (stat(file, &st) == -1)
+		return -1;
+	*time = st.st_mtime;
+	return 0;
+}
+
+/* Handy routine to read very long lines in text files.
+ * This means we read the whole line and avoid any nasty buffer overflows.
+ * We strip leading space and avoid comment lines, making the code that calls
+ * us smaller. */
+char *
+get_line(char ** __restrict buf, ssize_t * __restrict buflen)
+{
+	char *p, *c;
+	bool quoted;
+
+	do {
+		p = *buf;
+		if (*buf == NULL)
+			return NULL;
+		c = memchr(*buf, '\n', (size_t)*buflen);
+		if (c == NULL) {
+			c = memchr(*buf, '\0', (size_t)*buflen);
+			if (c == NULL)
+				return NULL;
+			*buflen = c - *buf;
+			*buf = NULL;
+		} else {
+			*c++ = '\0';
+			*buflen -= c - *buf;
+			*buf = c;
+		}
+		for (; *p == ' ' || *p == '\t'; p++)
+			;
+	} while (*p == '\0' || *p == '\n' || *p == '#' || *p == ';');
+
+	/* Strip embedded comments unless in a quoted string or escaped */
+	quoted = false;
+	for (c = p; *c != '\0'; c++) {
+		if (*c == '\\') {
+			c++; /* escaped */
+			continue;
+		}
+		if (*c == '"')
+			quoted = !quoted;
+		else if (*c == '#' && !quoted) {
+			*c = '\0';
 			break;
 		}
 	}
-	fclose(fp);
-	return len;
+	return p;
 }
 
+
 int
 is_root_local(void)
 {
