@@ -193,6 +193,17 @@ if_setflag(struct interface *ifp, short setflag, short unsetflag)
 	return 0;
 }
 
+bool
+if_is_link_up(const struct interface *ifp)
+{
+
+	return ifp->flags & IFF_UP &&
+	    (ifp->carrier == LINK_UP ||
+	     (ifp->carrier == LINK_UNKNOWN &&
+	      !(ifp->options == NULL ||
+	        ifp->options->options & DHCPCD_LINK)));
+}
+
 int
 if_randomisemac(struct interface *ifp)
 {
@@ -411,11 +422,16 @@ if_check_arphrd(struct interface *ifp, unsigned int active, bool if_noconf)
 		}
 		break;
 	default:
-		if (if_noconf)
-			active = IF_INACTIVE;
-		if (active)
-			logwarnx("%s: unsupported interface type 0x%.2x",
+		if (active) {
+			int i;
+
+			if (if_noconf)
+				active = IF_INACTIVE;
+			i = active ? LOG_WARNING : LOG_DEBUG;
+			logmessage(i, "%s: unsupported"
+			    " interface type 0x%.2x",
 			    ifp->name, ifp->hwtype);
+		}
 		break;
 	}
 
@@ -621,12 +637,14 @@ if_discover(struct dhcpcd_ctx *ctx, struct ifaddrs **ifaddrs,
 #endif
 			default:
 				/* Don't allow unless explicit */
-				if (if_noconf)
-					active = IF_INACTIVE;
-				if (active)
-					logwarnx("%s: unsupported"
+				if (active) {
+					if (if_noconf)
+						active = IF_INACTIVE;
+					i = active ? LOG_WARNING : LOG_DEBUG;
+					logmessage(i, "%s: unsupported"
 					    " interface type 0x%.2x",
 					    ifp->name, sdl->sdl_type);
+				}
 				/* Pretend it's ethernet */
 				ifp->hwtype = ARPHRD_ETHER;
 				break;
